/*
   $Id: UInewt.c,v 1.9 2001/11/04 13:36:36 pawelk Exp $
   Author: Pawe Koodziej <pawelk@pld.org.pl>
 */

#include "config.h"
#include <stdio.h>
#include <stdlib.h>
#include <rpm/rpmlib.h>
#include <dirent.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <sys/stat.h>
#include <errno.h>
#include <newt.h>
#include <newt-addon.h>

#include "rpmmen.h"
#include "pkgssel.h"
#include "rpmmen_config.h"
#include "rpmpldi.h"
#include "module.h"
#include "wuch.h"


extern char *pkgsListFname;

#define REDIRECT_STDOUT redirect_out(1)
#define RESTORE_STDOUT  redirect_out(0)


#if 0
#define select_from_file pldi_select_from_file
#define rpmmen_install pldi_rpmmen_install
#define rpmmen_scandir pldi_rpmmen_scandir
#endif

struct install_ui_tag {
	newtComponent form;
	newtComponent pkg_lbl;
	newtComponent pkg_scale;
	int pkg_scale_v;
	int npkgs;
	int ninstpkgs;
	newtComponent pkgs_lbl;
	newtComponent pkgs_scale;
	unsigned long int pkgs_instsize;
	unsigned long int pkgs_size;
};

static struct install_ui_tag install_ui;
static const char *stdoutfile = RPMMEN_STDOUT_FILE;
static int excludeDoc;	/* ustawione jeli instalujemy bez dokumentacji */


static void set_pkgs_label(void) 
{
	if (install_ui.pkgs_instsize) {
		char lbl[128];
		unsigned long prec = (install_ui.pkgs_instsize * 10) / install_ui.pkgs_size;
		prec *= 10; 
		log_debug("prec %ld * 100 / %ld = %ld", install_ui.pkgs_instsize,
				install_ui.pkgs_size, prec);
		snprintf(lbl, sizeof(lbl),
				_("%ld%% installed (%d of %d packages, %ld of %ld KBytes)"),
				prec, 
				install_ui.ninstpkgs, install_ui.npkgs,
				install_ui.pkgs_instsize/1000, install_ui.pkgs_size/1000);
		newtLabelSetText(install_ui.pkgs_lbl, lbl);
	}

}

/* redirect_out - if( to >0  ) redirect stdout and stderr to file, if (!to)
 * redirect to console. if (to < 0)  redirect to console, and close file.
 */
void redirect_out(int to)
{
	static int new = -1, o2 = -1, o1 = -1;

	if (new < 0) {
		new = open(stdoutfile, O_RDWR | O_CREAT, 0644);

		log_debug("redirect_out: file handle: %d", new);
		if (new == -1) {
			log_message(_("can't open file for stdout "
					"(not good, but not critical) : %s"),
					strerror(errno));
			return;
		}
	}
	if (o1 == -1) {
		o2 = dup(2);
		o1 = dup(1);
	}
	if (to > 0) {
		dup2(new, 2);
		dup2(new, 1);
	}
	if (to == 0) {
		dup2(o2, 2);
		dup2(o1, 1);
	}
	if (to < 0) {
		redirect_out(0);
		close(new);
	}
}


/* zmie napisy w oknie z postpem instalacji na dane nowego pakietu */
void UIOpenPackage(const TPkgInfo *pkg)
{
	char label[1024];

	RESTORE_STDOUT;

	if (pkg == NULL)
		strncpy(label, _("Preparing to installation..."), sizeof(label));
	else 
		snprintf(label, sizeof(label), _("Installing package %s"), pkg->name);

	newtLabelSetText(install_ui.pkg_lbl, label);
	newtScaleSet(install_ui.pkg_scale, 0); /* Skala od 0 do 100 bo nie mog
						  zmieni dynamicznie zakresu skali*/
	install_ui.pkg_scale_v = 0;
	newtScaleSet(install_ui.pkgs_scale, install_ui.pkgs_instsize);
	set_pkgs_label();
	newtRefresh();
	REDIRECT_STDOUT;
}


void UIInstallProgress(const unsigned long amount, const unsigned long total)
{
	int v;

	if (amount && amount == total) /* last notification */
		install_ui.pkgs_instsize += total;

	v = amount * 100 / total;
	if (v != install_ui.pkg_scale_v) {
		install_ui.pkg_scale_v = v;
		RESTORE_STDOUT;
		newtScaleSet(install_ui.pkg_scale, v);
		newtRefresh();
		REDIRECT_STDOUT;
	}
}


void UIClosePackage(const TPkgInfo *pkg)
{
	log_debug("Close package %s", pkg->name);
	install_ui.ninstpkgs++;
}

void UIrpmmen_deps(int ac, int pos, int n_max, const char *label)
{
	static newtComponent f, l1, l2;
	static int max = 0;
	char tmp[1000];


	if (n_max)
		max = n_max;
		
	switch (ac) {
		case 0:
			RESTORE_STDOUT;
			if (f) {
				newtFormDestroy(f);
				newtPopWindow();
				newtRefresh();
				f = 0;
			}
			newtCenteredWindow(42, 4,
					_("Adding packages required by other"));
			f = newtForm(NULL, NULL, 0);
			l1 = newtLabel(2, 1, "");
			l2 = newtLabel(8, 2, "");
			newtFormAddComponents(f, l1, l2, NULL);
			newtDrawForm(f);
			REDIRECT_STDOUT;
		case 1:
			RESTORE_STDOUT;
			if (label)
				newtLabelSetText(l1, label);
			sprintf(tmp, _("%6d/%-6d"), pos, max);
			newtLabelSetText(l2, tmp);
			newtRefresh();
			REDIRECT_STDOUT;
			break;
		case 2:
			RESTORE_STDOUT;
			if (f) {
				newtFormDestroy(f);
				newtPopWindow();
				newtRefresh();
				f = 0;
			}
			break;
	}
}


TPkgInfo * UIselectOne(struct rpmDependencyConflict *conflict)
{
	newtComponent f, l1,l2, list,b1;
	char buf[100];
	int i;
	void *ret;
	RESTORE_STDOUT;
	newtCenteredWindow(62, 14,_("Select one package"));
	f = newtForm(NULL, NULL, 0);
	sprintf(buf,"Package '%s' needs", conflict->byName);
	l1 = newtLabel(2, 1, buf);
	sprintf(buf, "'%s' %s.  It's provided by packages:",
			conflict->needsName, 
			conflict->needsVersion);
	l2 = newtLabel(2, 2,buf);
			
	b1 = newtButton(1,10,"OK, install this one");
	list = newtListbox(2, 4,5,0);
	for(i=0; conflict->suggestedPackages && conflict->suggestedPackages[i];
			i++){
		sprintf(buf,"%-40s",
			((TPkgInfo *)conflict->suggestedPackages[i])->name);
		newtListboxAppendEntry(list, buf,
			   conflict->suggestedPackages[i]);
	}
	
	newtFormAddComponents(f, l1,l2,list, b1 , NULL);

	newtRunForm(f);
	ret = newtListboxGetCurrent(list);
	newtFormDestroy(f);	
	newtPopWindow();
	REDIRECT_STDOUT;
	return ret;

}	

/**
 * \return static buffer  !!!
 */
char *mkstrNVRF(char *name,char *ver,char *rel,int flag)
{
	static char ret[100];

	strcpy(ret,name);
	if(ver&&strlen(ver)){
		strcat(ret," (");
		if(flag&RPMSENSE_LESS)
			strcat(ret,"<");
		if(flag&RPMSENSE_EQUAL)
			strcat(ret,"=");
		if(flag&RPMSENSE_GREATER)
			strcat(ret,">");
		strcat(ret,ver);
		if(rel){
			strcat(ret,"-");
			strcat(ret,rel);
		}
		strcat(ret,")");
	}
	return ret;
}

int UIcb_conflict(int action,char *name1,char *ver1,char *rel1,
		char *name2,char *ver2,int flag2)
{
	static char  *ctext=NULL,*utext=NULL;
	newtComponent form,textbox,btOK;
	char *uhead=_("\n\nPackages below requires some features which are\n"
			"not provided by any other package.\n");
	char *chead=_("Packages below can't be installed simultaneously.\n"
			"You have to manually deselect some of them.\n");
	char *text;
	int size;

	switch(action){
		case RPMMEN_CB_CONFLICT_ADD:
			if(!ctext)
				ctext=strdup("");
			ctext=(char *)realloc(ctext,strlen(ctext)+1+
					strlen(name1)+strlen(name2)+80);
			strcat(ctext,"\n");
			strcat(ctext,mkstrNVRF(name1,ver1,rel1,0));
			strcat(ctext,"  <--->  ");
			strcat(ctext,mkstrNVRF(name2,ver2,NULL,flag2));
			break;
		case RPMMEN_CB_CONFLICT_UNSATISFIED:
			if(!utext)
				utext=strdup("");
			utext=(char *)realloc(utext,strlen(utext)+1+
					strlen(name1)+strlen(name2)+80+
					strlen(_("\npackage "))+
					strlen(_("  requires ")));
			strcat(utext,_("\npackage "));
			strcat(utext,mkstrNVRF(name1,ver1,rel1,0));
			strcat(utext,_("  requires "));
			strcat(utext,mkstrNVRF(name2,ver2,NULL,flag2));
			break;
		case RPMMEN_CB_CONFLICT_NEW:
			if(ctext)
				free(ctext);
			ctext=NULL;
			if(utext)
				free(utext);
			utext=NULL;

			break;
		case RPMMEN_CB_CONFLICT_SHOW:
			if((!ctext) && (!utext))
				return 0;
			newtCenteredWindow(60,20,_("Dependencies problems"));
			form=newtForm(NULL,NULL,0);
			size=0;
			if(ctext)
				size+=strlen(ctext)+strlen(chead);
			if(utext)
				size+=strlen(utext)+strlen(uhead);
			text=(char *)malloc(size+10);
			strcpy(text,"");
			if(ctext){
				strcat(text,chead);
				strcat(text,ctext);
			}
			if(utext){
				strcat(text,uhead);
				strcat(text,utext);
			}

			textbox=newtTextbox(2,2,50,12,
					NEWT_TEXTBOX_WRAP|NEWT_TEXTBOX_SCROLL);
			newtTextboxSetText(textbox,text);
			btOK=newtButton(20,16,_("OK"));

			newtFormAddComponents(form,textbox,btOK,NULL);
			newtRunForm(form);
			newtPopWindow();

			return 1;
	}
	return 0; /* if function return heare returned value is _not_ 
		     important */
}

int UIdoInstall()
{
	int again;
	statusMessage(NULL, _("Preparing to installation..."));
	//    rpmpldi_add_deps();
	RESTORE_STDOUT;
	destroy_statusMessage();
#define WINWIDTH 66    
	newtCenteredWindow(WINWIDTH, 10, _("Packages installation"));
	memset(&install_ui, 0, sizeof(install_ui));
	install_ui.form = newtForm(NULL, NULL, 0);
	install_ui.pkg_lbl = newtLabel(2, 1, _("Preparing to installation..."));
	install_ui.pkg_scale = newtScale(2, 3, WINWIDTH - 4, 100);
	install_ui.pkg_scale_v = 0;

	install_ui.pkgs_size = rpmpldi_selected_size(&install_ui.npkgs);
	log_debug("selsize = %ld", install_ui.pkgs_size);

	install_ui.pkgs_lbl = newtLabel(2, 5, "");
	install_ui.pkgs_scale = newtScale(2, 6, WINWIDTH - 4, install_ui.pkgs_size);
	install_ui.pkgs_instsize = 0;
	install_ui.ninstpkgs = 0;
	set_pkgs_label();

	newtFormAddComponents(install_ui.form, install_ui.pkg_lbl,
			install_ui.pkg_scale, NULL);
	newtDrawForm(install_ui.form);
	newtRefresh();

	install_ui.pkgs_size = rpmpldi_selected_size(&install_ui.npkgs);

	REDIRECT_STDOUT;
	rpmpldi_install(excludeDoc ? RPMTRANS_FLAG_NODOCS : 0);
	RESTORE_STDOUT;
	newtFormDestroy(install_ui.form);
	newtPopWindow();
	newtRefresh();
	again=newtWinChoice(_("Again ?"),_("Yes (again)"),_("No (quit)"),
			_("Do you want install/upgrade other packages ?"))==1;

	return again;

}


int UIsaveList(){
	TPkgSet *pkgs;
	TPkgInfo *pk;
	FILE *f;
	int i;

	f=fopen("wuch.list","w");
	if(f==NULL){
		newtWinMessage("Error","ok","Can't opem file for writing.\nPlease check it and try again");
		return 2;
	}
	pkgs=rpmpldi_getPkgsSet();
	/* deselect all packages */
	for(i=0;i<pkgs_array_num(pkgs);i++){
		pk = pkgs_array_nth(pkgs,i);
		if(pk->selected)
			fprintf(f,"%s\n",pk->name);
	}
	fclose(f);
	newtWinMessage("Success","ok","Packages list saved.");
	return 0;
}
/** run rpms instalation (with user interface).

  \param names -- names of packages which should be preselected
  \param count --count of names
  \return -1 - ok - instalaction aborted by user <br>
  \return 0   - ok - rpms installation successful <br>
  \return  1  - destantion media error <br>
  \return 2  - source media error 

 */
int rpmmenUI_run(char *src, char *dest, const char *stdoutf,
		char **names, int count, int mode)
{
	
	
	int i, err = 0,rc;
	struct rpmmen_cb cb;
	int again;
	char message[1000];

	if (stdoutf)
		stdoutfile = stdoutf;

	/* rpmlib prints all instalation's errors on stderr. So now i create
	 * file to which it will be redirected
	 */
	statusMessage(NULL, _("Building list of available packages..."));


	cb.pkg_open = UIOpenPackage;
	cb.pkg_inst_progress = UIInstallProgress;
	cb.pkg_close = UIClosePackage;
	cb.deps_prog = UIrpmmen_deps;
	cb.conflict  = UIcb_conflict;
	cb.selectOne = UIselectOne;

	REDIRECT_STDOUT;    
	i = rpmpldi_init(src, dest, &cb);
	if (i)
		err = 1;

	RESTORE_STDOUT;

	switch (i) {

		case 0:
			log_message(_("RPM base opened"));
			break;

		case 1:
			log_message(_("Can't create dierctory for rpm base"));
			newtWinMessage(_("Error"), _("OK"),
					_("Could not create directory for RPM base on "
						"your destantion media.\n"
						"Please correct this problem and try agin."));
			break;

		case 2:
			log_message(_("RPM base not opened"));
			newtWinMessage(_("Error"), _("OK"),
					_("Could _not_ open (create) RPM database.\n"
						"It means that you have some problems with "
						"destantion drive.\n"
						"Please check it and try agin"));
			break;

		default:
			log_message(_("%s default section in switch"), __FUNCTION__);
			newtWinMessage(_("Error"), _("OK"),
					_("Congratulations"
						"\n\nYou found bug in PLD Installer\n"
						"please don't forget announce it !!!\n"
						"Anyway you probably have some problem with "
						"destantion media.\n Check it and try agin"));
			err = 1;
			break;
	}

	if (err) {
		RESTORE_STDOUT;
		destroy_statusMessage();
		goto end;
	}
	REDIRECT_STDOUT;

	if (rpmpldi_scandir()) {
		RESTORE_STDOUT;
		log_message(_("Can't get list of packages from source media"));
		newtWinMessage(_("Error"), _("OK"),
				_("Installer could not "
					"got list of files from your source media.\n"
					"Please correct it, and try again"));
		err = 2;
	}

	if (err) {
		RESTORE_STDOUT;
		destroy_statusMessage();
		goto end;
	}

	//	rpmpldi_select_from_file(RPMMEN_BASEPKGSLIST);

	if(pkgsListFname)
		rpmpldi_select_from_file(pkgsListFname);
	rpmpldi_selectpackages(names,count);
	RESTORE_STDOUT;
	destroy_statusMessage();
	/*
	   if (pkgsSelector_init()) 
	   do{
	   if (!pkgsSelector())
	   err = -1;
	   if(err)
	   goto end;
	   }while(rpmpldi_add_deps());
	 */

	again=0;
	do{

		if(again==1){    /* second or higher iteration */
			TPkgSet *pkgs;
			pkgs=rpmpldi_getPkgsSet();
			/* deselect all packages */
			if(rc!=PKGSSEL_REREAD_DB)
				for(i=0;i<pkgs_array_num(pkgs);i++)
					pkgs_array_nth(pkgs,i)->selected=0;
			/* upgrade packages flags an "installed package version"
			 * information */
			statusMessage(NULL, _("Building list of available packages..."));
			rpmpldi_update_installed_info();
			destroy_statusMessage();
		}

		if (pkgsSelector_init()) {
			rpmpldi_closedb();
			rc=pkgsSelector(mode);
			rpmpldi_opendb();
			if(!rc){
				err=-1;
				goto end;
			}
		}

		pkgsSelector_finish();
		if(rc==PKGSSEL_REREAD_DB){
			again=1;
			continue;
		}

		if(mode==MODE_LIST){
			again=0;
			UIsaveList();
		}else
			again = UIdoInstall();

	}while(again);
		
		
	if(mode != MODE_LIST){

		sprintf(message,_("Output of all packages' instalation scripts has been"
					" saved in file %s"),stdoutfile);
		newtWinMessage(_("Important"), _("OK, I will check it"),message);
	}


	/* stdoutfile  */


end:
	redirect_out(-1);
	rpmpldi_finish();	/* zakonczenie pracy z  rpmmen'em */
	return err;
}

/* module entry point */
void newt_ui(struct module_context *ctx)
{
	if (geteuid() != 0) {
		printf(_("This program is useful only when started "
					"with root priviliges !!!\n"));
		exit(1);
	}

	newtInit();
	newtCls();

	newtDrawRootText(0, 0, _("wuch v" VERSION));
	newtDrawRootText(-22, 0, _("PLD Linux Distribution"));
	rpmmenUI_run("", (char*)ctx->root, NULL, ctx->upgrade_list,
			ctx->upgrade_count, ctx->mode);
	newtFinished();
}
