/*
   PkgsSel-test
   Copyright (C) 1999 Jarek Woloszyn (yossa@pld.org.pl)
   $Id: ttest.c,v 1.1 2001/02/05 15:51:06 malekith Exp $
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the
   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <dirent.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <sys/stat.h>
#include <newt.h>
#include <pldi.h>
#include <rpmmen.h>
#include <rpm/rpmlib.h>
#include <errno.h>
#include <pkgssel.h>
#include <rpmmen_config.h>
#include <rpmpldi.h>

/* linijka z postpem instalaji pakietu */
static void UIrpmmen_progress(const unsigned long act, const unsigned long all);

newtComponent frm, lblGl, scl1;

static int excludeDoc;		/* ustawione jeli instalujemy bez dokumentacji */

/* redirect_out - if( to >0  ) redirect stdout and stderr to file, if (!to)
 * redirect to console. if (to < 0)  redirect to console, and close file.
 */
void redirect_out(int to)
{
    static int new = -1, o1 = -1;

    if (new < 0) {
	new = open(RPMMEN_STDOUT_FILE, O_RDWR | O_CREAT, 0644);

	logDebugMessage("redirect_out: file handle: %d", new);
	if (new == -1) {
	    logDebugMessage("can't open file for stdout ",
			    "(not good, but not critical) : %s",
			    strerror(errno));
	    return;
	}
    }
    if (o1 == -1) {
	o1 = dup(1);
    }
    if (to > 0) {
	dup2(new, 1);
    }
    if (to == 0) {
	dup2(o1, 1);
    }
    if (to < 0) {
	redirect_out(0);
	close(new);
    }
}


/* zmie napisy w oknie z postpem instalacji na dane nowego pakietu */
void UIstartPackage(const TPkgInfo * kt, const unsigned long amount,
                    const unsigned long total)
{

    char tmp[1000] = "TO NIE POWINNO SIE POJAWIC !!!";
    if (kt == NULL)
	sprintf(tmp, _("Preparing to installation"));
    else {
	sprintf(tmp, _("Installing package %s"), kt->name);
	logDebugMessage(_("Installing package %s"), kt->name);
    }
    newtLabelSetText(lblGl, tmp);
    newtScaleSet(scl1, 0);	/* Skala od 0 do 100 bo nie mog
				   zmieni dynamicznie zakresu skali */
    newtDrawForm(frm);
    /*printf("@@@@@@%s@@@@@@\n",kt->name); */
}


void UIrpmmen_progress(const unsigned long amount, const unsigned long total)
{
    redirect_out(0);
    newtScaleSet(scl1, amount * 100 / total);
    newtDrawForm(frm);
    newtRefresh();
    redirect_out(1);
}

void UIrpmmen_deps(int ac)
{
    static newtComponent f, l1, l2;
    static int i = 0;
    static char klepsydry[] = "-\\|/";
    char tmp[1000];

    switch (ac) {
    case 0:
	newtCenteredWindow(38, 4,
			   _("Packages dependencies checking"));
	f = newtForm(NULL, NULL, 0);
	l1 = newtLabel(2, 1, _("Adding packages which are "
			       "required"));
	l2 = newtLabel(8, 2, "");
	newtFormAddComponents(f, l1, l2, NULL);
	break;
    case 1:
	sprintf(tmp, _("by selected packages %c"), klepsydry[i++ % 4]);
	newtLabelSetText(l2, tmp);
	newtDrawForm(f);
	newtRefresh();
	break;
    case 2:
	newtFormDestroy(f);
	newtPopWindow();
	newtRefresh();
	break;
    }

}



/* z kd, do kd instalow i czy z dokumentacj */
void get_src(char *src, char *dest)
{


    char *srcpath, *destpath;
    char inst_doc;
    newtComponent frm, btnOK, lblSrc, lblDest, entSrc, entDest, chkDoc;
    newtCenteredWindow(50, 8, _("Packages installation"));
    frm = newtForm(NULL, NULL, 0);
    btnOK = newtButton(37, 2, _("OK"));
    lblSrc = newtLabel(2, 1, _("Install from"));
    entSrc = newtEntry(2, 2, "/mnt/cdrom/RedHat/RPMS", 30, &srcpath, NEWT_ENTRY_SCROLL);
    lblDest = newtLabel(2, 4, _("to (no changes here, rather)"));
    entDest = newtEntry(2, 5, "/var/tmp/a", 30, &destpath, NEWT_ENTRY_SCROLL);
    chkDoc = newtCheckbox(2, 7, _("install with documentation"), '*', NULL, &inst_doc);
    newtFormAddComponents(frm, btnOK, lblSrc, lblDest, entSrc, entDest, chkDoc, \
			  NULL);
    newtRunForm(frm);

    strcpy(dest, destpath);
    strcpy(src, srcpath);

    if (inst_doc == ' ')
	excludeDoc = 1;
    logDebugMessage("excludeDoc: %d", excludeDoc);
    newtFormDestroy(frm);
    newtPopWindow();
    newtRefresh();
}


/* run rpms instalation (with user interface)
   RET: 0 - ok - rpms installation successful
   1 - destantion media error
   2 - source media error
 */


int rpmmenUI_run()
{

    int i;
    char src[1000], dest[1000];

    /* rpmlib prints all instalation's errors on stderr. So now i creat
     * file to which it will be redirected
     */
    redirect_out(1);

    redirect_out(0);
    get_src(src, dest);		/* XXX should be in srcsel */
    redirect_out(1);
    i = rpmpldi_init(src, dest, UIstartPackage, UIrpmmen_progress,
                     UIrpmmen_deps);
    if (i)
	rpmpldi_finish();
    
    redirect_out(0);

    switch (i) {
    case 0:
	logMessage("RPM base opened");
	break;
    case 1:
	logMessage("Can't creat dierctory for rpm base");
	newtWinMessage(_("Error"), _("OK"), _("Installator "
			     "could not creat directory for RPM base on "
					      "your destantion media.\n"
			   "Plesae correct this problem and try agin."));
	return 1;
    case 2:
	logMessage("RPM base not opened");
	newtWinMessage(_("Error"), _("OK"), _("Installator "
			      "could  _not_ open (creat) RPM database.\n"
			      "It mean that you have some problems with "
					      "destantion drive.\n"
					"Please check it and try agin"));
	return 1;
    default:
	logMessage("%s default section in switch",
		   __FUNCTION__);
	newtWinMessage(_("Error"), _("OK"), _("Congratulations"
				 "\n\nYou found bug in PLD installator\n"
				  "please don't forgot announce it !!!\n"
			  "Any way you probabbly have some problem with "
			   "destantion media.\n Check it and try agin"));
	return 1;
    }

    redirect_out(1);

    if (rpmpldi_scandir()) {
	redirect_out(0);
	newtWinMessage(_("Error"), _("OK"), _("Installator could not "
			    "got lits of files from your source media.\n"
				     "Please correct it, and try agin"));
	rpmpldi_finish();
	return 2;
    }
    redirect_out(1);
    rpmpldi_select_from_file("coinst.rpmy");

    redirect_out(0);
    pkgsSelector_init();
    pkgsSelector();
    pkgsSelector_finish();

/*      rpmpldi_add_deps();            */
    frm = newtForm(NULL, NULL, 0);
    lblGl = newtLabel(2, 1, "");
    scl1 = newtScale(2, 5, 46, 100);
    newtFormAddComponents(frm, lblGl, scl1, NULL);
    newtRefresh();
    redirect_out(1);
    rpmpldi_install((excludeDoc ? RPMTRANS_FLAG_NODOCS : 0)	/*, UIrpmmen_progress, \
								   UIstartPackage */ );
    redirect_out(-1);
    newtFormDestroy(frm);
    rpmpldi_finish();		/* zakonczenie pracy z  rpmmen'em */
    return 0;
}
int main()
{
    int old2, new2;
    new2 = open("instal.stderr", O_RDWR | O_CREAT, 0644);
    old2 = dup(2);
    dup2(new2, 2);
    log_open("debug.log");
    iointf_init("fs", NULL);
    iointf_chroot("/");
    newtInit();
    newtCls();
    newtDrawRootText(0, 0, _("Packages Instalation"));
    newtDrawRootText(-24, 0, _("Polish Linux Disribution"));

    while (rpmmenUI_run());
    newtFinished();
    iointf_done();
    log_close();

    dup2(2, old2);
    close(new2);

    return 0;
}
